/**	@file 
	@brief Functionality for the main @b NokSync window.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: main.js 27 2006-12-19 00:45:46Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(main)";


/** Global Thunderbird address book directory.
	Initialized by @c initMain();
	@type nsIAbDirectory
**/	var abDirectory;

/** The branch of the preference tree, in which NokSync preferences lives.
	Initialized by @c initMain();
	@type nsIPrefBranch
**/	var prefBranch;

/** Bundle of strings to be used on the GUI.
	Initialized by @c initMain();
	@type stringbundle
**/	var strings;

/** Preference name for do not show disclaimer
	@type string
**/	var prefNoDisclaimer="noksync.noDisclaimer";

/** Preference prefix for selected address books.
	Also defined in preferences.js.
	@type string
**/	var prefAbook="noksync.abook.";

/** Preference name for users supported phone
	@type string
**/	var prefSupported="noksync.supported.";


// ======================================================

/** When @b cancel is pressed, cancel() will set @c shouldCancel=true.
	Each function should check this, and cancel further actions if @c true.
	@type boolean
**/	var shouldCancel=false;

/** When a function has responed to a @c isCancelled,
	it should set @c isCancelled=true.
	When cancel is subsequently pressed again, cancel() will close the window.
	@type boolean
**/	var isCancelled=false;

/** When @b OK is pressed, @c execute sets @c isStarted=true.
	If cancel is pressed before @c isStarted=true,
	@c cancel simply closes the window.
	@type boolean
**/	var isStarted=false;


// ======================================================

/** Array of Thunderbird address books selected in preferences.
	Set by @c getAbooks();
	@type nsIAbDirectory[n]
**/	var aBooks;

/** Array of the @b names of the Thunderbird address books selected in preferences.
	Set by @c getAbooks();
	@type string[n]
**/	var aBookNames;

/** Global variable holding an array of thunderbird address cards
	Initialized by @c stepTBaddresses().
	@todo DESTROY 
	@type nsIAbCard[]
**/	var aAbCard=[];

/** Global variable holding an array of thunderbird addressbooks,
	matching @c aAbCard.
	Initialized by @c stepTBaddresses().
	@todo DESTROY 
	@type nsIAbDirectory[]
**/	var aCardAB=[];


// ======================================================

/** Global variable holding the error result of a call to a @b C++ function.
	Must be initialized to an @c Object, as it is an @c out parameter.
	The actual (@c long) value is found in @c hResult.value
	The only reason for keeping it global is to make sure it is initialized.
	@type Object
**/	var hResult=new Object();

/** Global variable holding the NMP error result of a call to a @b C++ function.
	Must be initialized to an @c Object, as it is an @c out parameter.
	The actual (@c long) value is found in @c nmpResult.value.
	The only reason for keeping it global is to make sure it is initialized.
	@type Object
**/	var nmpResult=new Object();


// ======================================================

/** Global constant defining the number of times to try
	to connect to the NOKIA phone.
	@type integer
**/	var nConnect=100;

/** Global variable holding the number of times we have tried
	to connect to the NOKIA phone.
	Used by @c stepNokiaConnect...
	@type integer
**/	var iConnect;

/** Global constant defining the number of milliseconds 
	to wait between each try to connect to the NOKIA phone.
	@type integer
**/	var msWait=450;

/** Global variable holding reference to NOKIA phone information.
	Initialized by @c stepNokiaConnect().
	Destroyed by @c stepNokiaConnectDone().
	@type INokiaPhoneInfo
**/	var nokiaPhoneInfo;

/** Global variable holding reference to NOKIA phone book.
	Initialized by @c stepNokiaReadContacts().
	@todo DESTROY 
	@type INokiaPhonebook
**/	var nokiaPhonebook;

/** Global variable holding an array of NOKIA contacts.
	Initialized by @c stepNokiaReadContacts...().
	@todo DESTROY 
	@type INokiaContact[]
**/	var aNokiaContact=[];

/** Global variable holding the number of available NOKIA contacts.
	Initialized by @c stepNokiaReadContacts().
	@type integer
**/	var nContact;

/** Global variable holding the total number of contacts to synchronize.
	Initialized by @c stepSynchronize().
	@type integer
**/	var nContacts;

/** Global variable holding the index of the current
	NOKIA contact to be read.
	Used by @c stepNokiaReadContacts...().
	@type integer
**/	var iContact;


// ======================================================

/** Global variable holding the comparison between Thunderbird and NOKIA.
	See @ref Comparator.js_.
	Initialized by @c stepSelectContactsCompare
	and used in @c stepSelectContactsCompareWin and hence in the @c SyncSelection window.
	@type Comparison
**/ var comparison;

/** Global variable holding the RDF data source from the @c SyncSelection window.
	Initialized in @c stepSelectContactsWin as return value from opening @c SyncSelection.xul.
	Used in @c stepSynchronizeXXX, which checks @c properties of @c dsTree,
	to check if each contact should be synchronized.
	@type nsIRDFDataSource
**/ var dsTree;

/** Global Was phone originally thought as unsupported?
	@type bool
**/	var unsupported=false;

/** Global The user's phone's product type
	@type bool
**/	var productType;

/** Global array of product types supported by @b NokSync.
	@type String[]
**/var supportedProductTypes=[
	// Officially supported by Nokia PC Connectivity Suite 3.0
	"NPE-3", // Nokia 6210
	"NHM-3", // Nokia 6250
	"NSE-5", // Nokia 7110
	"NSW-5", // Nokia 7160
	"NSB-5", // Nokia 7190
	
	// The authors phone...
	"RM-78", // Nokia 6280
	
	// Reported by "casteels" at http://sourceforge.net/forum/message.php?msg_id=4063763
	"RM-14", // Nokia 6233
	];


// ======================================================
//		HELPER FUNCTIONS
// ======================================================

/** Check if NOKIA phone with given product type is supported by @b NokSync.
	@tparam string pt
	@type boolean
**/
function checkProductType(pt) {
	for (var i in supportedProductTypes)
		if (pt==supportedProductTypes[i]) return true;
	return false;
	}


// ======================================================
/** Find address books in Thunderbird.
	Set global @c aBooks to array of nsIAbDirectory,
	and set global @c aBookNames to array of the names of the addressbooks.
	@type void
**/
function getAbooks() {
	try {	
		log(1, "Looking for address-book preferences");
		aBooks=[];
		aBookNames=[];
		var cn = abDirectory.childNodes;
		while (cn.hasMoreElements() ) {
			var ab = cn.getNext();
			var curDir = ab.QueryInterface(Components.interfaces.nsIAbDirectory);
			var abName=curDir.dirName;
  			log(2, "child dirName: " + abName);
			var val=false;			
			if (prefBranch.getPrefType(prefAbook+abName) == prefBranch.PREF_BOOL)
				val=prefBranch.getBoolPref(prefAbook+abName);
  			if (val) aBooks.push(ab);
  			if (val) aBookNames.push(abName);
		}
		log(1, "Number of selected aBooks found: " + aBooks.length);
	} catch (e) {
		error("Unable to get address books", e);
	    close(); return;
	}
	}


// ======================================================
/** Set icon and prompt to indicate that operation was cancelled.
	Also set global @c isCancelled to @c true.
	@tparam string step
	@type void
**/
function handleCancelled(step) {
			isCancelled=true;
			document.getElementById("icon"+step).setAttribute("class", "step-cancel");
			document.getElementById('prompt')
				.setAttribute("value", strings.getString("prompt.abortedLabel") );
	}


// ======================================================
/** Set class to show "working" icon for step 
	(Icon identified in @c noksync.css)
	@tparam string step
	@type void
**/
function setStepWorking(step) {
	document.getElementById("icon" + step).setAttribute("class", "step-working");
	}

// ======================================================
/** Set class to show "OK" icon for step 
	(Icon identified in @c noksync.css)
	@tparam string step
	@type void
**/
function setStepOk(step) {
	document.getElementById("icon" + step).setAttribute("class", "step-ok");
	}

// ======================================================
/** Set class to show "Error" icon for step 
	(Icon identified in @c noksync.css)
	@tparam string step
	@type void
**/
function setStepError(step) {
	document.getElementById("icon" + step).setAttribute("class", "step-error");
	}
	
// ======================================================
/** Set class to show "Cancelled" icon for step 
	(Icon identified in @c noksync.css)
	@tparam string step
	@type void
**/
function setStepCancel(step) {
	document.getElementById("icon" + step).setAttribute("class", "step-cancel");
	}

// ======================================================
/** Set label for step 
	@tparam string step
	@tparam string label
	@type void
**/
function setStepLabel(step, label) {
	document.getElementById("label"+step)
			.setAttribute("value", strings.getString(label) );
	}

// ======================================================
/** Set label for step; postfix count 
	@tparam string step
	@tparam string label
	@tparam int n
	@type void
**/
function setStepLabelN(step, label, n) {
	document.getElementById("label"+step)
			.setAttribute("value", strings.getString(label) + " " + String(n) );
	}

// ======================================================
/** Set label for step; put @c n in label by @c getFormattedString
	@tparam string step
	@tparam string label
	@tparam int n
	@type void
**/
function setStepLabelF(step, label, n) {
	document.getElementById("label"+step)
			.setAttribute("value", strings.getFormattedString(label, [String(n)]) );
	}

    
// ======================================================
/** Open the preferences window
	@type void
**/
function openPreferences() {
	log(1, "Opening preferences");
    window.open("chrome://noksync/content/preferences.xul", "", "chrome,centerscreen");
    }
    
// ======================================================
/** Open the @c SyncSelection window
	@type void
**/
function openSyncSelection() {
	log(1, "Opening SyncSelection");
    window.open("chrome://noksync/content/syncSelection.xul", "", "chrome,centerscreen,resizable");
    }


// ======================================================
//		INITIALIZATION
// ======================================================

/** Call the @c nokSyncLib initialization.
	Then initialize @c AbDirectory, @c PrefBranch, and @c strings, i.e. @c noksync-strings.
	Finally, set a timeout to start verifying the environment.
	@type void
**/
function initMain() {
	init();
	
	try {
		log(1, "Initializing(Main)");
		
  		log(2, "noksync-strings");
		strings = document.getElementById("noksync-strings");

  		log(2, "AbDirectory");
		abDirectory = Components.classes["@mozilla.org/rdf/resource-factory;1?name=moz-abdirectory"]
							.getService(Components.interfaces.nsIAbDirectory);
  		log(2, "PrefBranch");
  		prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

		window.setTimeout(showDisclaimer, 0);
		log(1, "Initialized(Main)");
	} catch (e) { errorClose2("Error while initializing", e); }                    
}

// ======================================================
/** Show disclaimer window
	@type void
**/
function showDisclaimer() {
	try {
		log(1, "showDisclaimer");
		var f=false;
		if (prefBranch.getPrefType(prefNoDisclaimer) == prefBranch.PREF_BOOL)
			f=prefBranch.getBoolPref(prefNoDisclaimer);
		if (!f) {
			window.openDialog("chrome://noksync/content/disclaimer.xul", "", "chrome,centerscreen,modal,dialog");
			prefBranch.setBoolPref(prefNoDisclaimer, true);
			}
		log(1, "showDisclaimer done");
		window.setTimeout(verifyEnvironment, 0);
	} catch (e) { errorClose2("Error while showing disclaimer", e); }                    
}

// ======================================================
/** Verify Environment.
	Verify that
	- we are running on windows
	- NOKIA PC Suite is installed
	- Nokia Phonebook Adapter is available
	Otherwise close the window!
	@type void
**/
function verifyEnvironment() {
	try {
		log(1, "verifyEnvironment");
		var flags=promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_0 +
				promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;

		// Warn if we are not running on windows
  		log(1, "Verify Windows");
		var runtime = Components.classes["@mozilla.org/xre/app-info;1"]
							.getService(Components.interfaces.nsIXULRuntime);
		log(1, "OS=" + runtime.OS);
		if (runtime.OS.substring(0,3) != "WIN" ) {
			var reply = promptService.confirmEx(window,
						strings.getString("nonwindows.title"),
						strings.getString("nonwindows.opsys") + " " + runtime.OS + "\n" +
						strings.getString("nonwindows.blurb"),
						flags,
						null, strings.getString("nonwindows.continue"), null,
						null, {});
			if (reply != 1) window.close();
			log(0, "Continuing with non-Windows operating system!");
			}
  
  		log(1, "Verify NOKIA PC Suite");
		var wrk = Components.classes["@mozilla.org/windows-registry-key;1"]
                    .createInstance(Components.interfaces.nsIWindowsRegKey);
		try {

	  		log(1, "Verify Nokia Phonebook Adapter");
			try {
				wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,
						"SOFTWARE\\Nokia\\PCSuite",
						wrk.ACCESS_READ);
				wrk.close();
				wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,
						"SOFTWARE\\Classes\\CLSID\\{FAB877D8-D79F-11D1-99A5-0060979AC1B6}",
						wrk.ACCESS_READ);
				wrk.close();
				}
			catch (e) {
				Components.utils.reportError(where + ": Nokia Phonebook Adapter verification failed: " + e);
				var reply = promptService.confirmEx(window,
							strings.getString("nopcsuitelib.title"),
							strings.getString("nopcsuitelib.blurb1") + "\n" + strings.getString("nopcsuitelib.blurb2"),
							flags,
							null, strings.getString("nopcsuitelib.continue"), null,
							null, {});
				if (reply != 1) window.close();
				log(0, "Continuing without Nokia Phonebook Adapter verified!");
				}

			}
		catch (e) {
			Components.utils.reportError(where + ": PC Suite verification failed: " + e);
			var reply = promptService.confirmEx(window,
						strings.getString("nopcsuite.title"),
						strings.getString("nopcsuite.blurb1") + "\n" + strings.getString("nopcsuite.blurb2"),
						flags,
						null, strings.getString("nopcsuite.continue"), null,
						null, {});
			if (reply != 1) window.close();
			log(0, "Continuing without NOKIA PC Suite verified!");
			}

  		log(1, "Verify NokSync DLL");
		var c=Components.classes["@KaarPoSoft.com/NokSync/NokiaPhoneInfo;1"];
		if (!c) {
			Components.utils.reportError(where + ": NokSync DLL verification failed.");
			var reply = promptService.confirmEx(window,
						strings.getString("nonoksyncdll.title"),
						strings.getString("nonoksyncdll.blurb1") + "\n" + strings.getString("nonoksyncdll.blurb2"),
						flags,
						null, strings.getString("nonoksyncdll.continue"), null,
						null, {});
			if (reply != 1) window.close();
			log(0, "Continuing without NokSync DLL verified!");
			}

		log(1, "verifyEnvironment done");
	} catch (e) { errorClose2("Error while verifying environment", e); }                    
}

// ======================================================
//		CALLBACKS
// ======================================================

/** Cancel callback.
	Called when cancel button is pressed.
	Just remember that cancel was pressed, and let the step handle the actual cancellation.
	If operation was not started at all, or cancel was already handled, then close window.
	@type void
**/
function cancel() {
	shouldCancel=true;
	//document.getElementById('cancel').setAttribute("disabled", true);
	if (!isStarted) window.close();
	if (isCancelled) window.close();
}


// ======================================================
/** Help callback.
	Called when help button is pressed
	@type void
**/
function help() {
	openURL("http://www.KaarPoSoft.dk/noksync/manual.html");
	}

// ======================================================
/** Report
	@type void
**/
function reportSupport() {
	openURL("http://sourceforge.net/forum/forum.php?forum_id=644284");
	}

	
// ======================================================
/** Execute callback.
	Called by @c addEventListener on @c load of window
	@type void
**/
function execute() {
	try {	

		document.getElementById('ok').setAttribute("disabled", true);
		document.getElementById('prompt')
					.setAttribute("value", strings.getString("prompt.runningLabel") );
		isStarted=true;
		window.setTimeout(stepPreferences, 0);
	} catch (e) { errorClose2("Error in starting synchronization", e); }                    
	}


		
// ======================================================
//		EVERY STEP I TAKE
// ======================================================

/** 
	Step 1: Get address-books from preferences
	@type void
**/
function stepPreferences() {
	try {
		setStepWorking("Preferences");
		getAbooks();
		if (aBooks.length==0) {
			log(1, "Initially no address-books; opening preferences");
			document.getElementById('labelPreferences')
					.setAttribute("value", strings.getString("stepPreferences.selectInPreferences") );
			
			window.openDialog("chrome://noksync/content/preferences.xul", "", "chrome,centerscreen,modal,dialog,dependent");
			getAbooks();
			}
					
		if (aBooks.length==0) {
			log(1, "Finally no address-books");

		document.getElementById('labelPreferences')
				.setAttribute("value", strings.getString("stepPreferences.noAbooks") );
				document.getElementById('prompt')
						.setAttribute("value", strings.getString("prompt.errorLabel") );

			setStepError("Preferences");
			isCancelled=true;
			return;
			}
		document.getElementById('labelPreferences')
			.setAttribute("value", strings.getString("stepPreferences.foundAbooks")+ " " + aBookNames);
		setStepOk("Preferences");

		window.setTimeout(stepTBaddresses, 0);
			
	} catch (e) { errorClose2("Error in getting address-books from preferences", e); }                    
	}


// ======================================================
/** 
	Step 2: Get addresses from Thunderbird
	@type void
**/
function stepTBaddresses() {
	try {
		if (shouldCancel) {
			handleCancelled("Preferences");
			return; }
			
		setStepWorking("TBaddresses");

		for (i in aBooks) {
			var aBook=aBooks[i];
			log(1, "aBook: " + aBook);
			log(1, "fileName: " + aBook.dirName);
			var cc=aBook.childCards;
			// childCards has nsIEnumerator interface, which is a pain...
			var more=false;
			var c;
			if (cc) {
				try {
					cc.first();
					c=cc.currentItem().QueryInterface(Components.interfaces.nsIAbCard);
					aCardAB.push(aBook);
					aAbCard.push(c);
					log(2, "FROM TBIRD: " + c.displayName);
					more=true
				} catch (c) {}
				
				while (more) {
					more=false;
					try {
						cc.next();
						c=cc.currentItem().QueryInterface(Components.interfaces.nsIAbCard);
						aCardAB.push(aBook);
						aAbCard.push(c);
						log(2, "FROM TBIRD: " + c.displayName);
						more=true;
					} catch (c) {}
				}
			}
		}

		document.getElementById('labelTBaddresses')
				.setAttribute("value", strings.getFormattedString("stepTBaddresses.foundAddresses", [String(aAbCard.length)]) );
		setStepOk("TBaddresses");

		window.setTimeout(stepNokiaConnect, 0);

	} catch (e) { errorClose2("Error in getting addresses from Thunderbird", e); }                    
	}


// ======================================================
/** 
	Step 3: Connect to phone
	@type void
**/
function stepNokiaConnect() {
	try {
		if (shouldCancel) {
			handleCancelled("TBaddresses");
			return; }

		setStepWorking("NokiaConnect");

		window.setTimeout(stepNokiaConnectConnect, 0);

	} catch (e) { errorClose2("Error in connecting to phone (init)", e); }
	}
	

// ======================================================
/** 
	Step 3xxx: Connect to phone - do the first connect
	@type void
**/
function stepNokiaConnectConnect() {
	try {
		if (shouldCancel) {
			handleCancelled("NokiaConnect");
			return; }

		try {
			var c=Components.classes["@KaarPoSoft.com/NokSync/NokiaPhoneInfo;1"];
			nokiaPhoneInfo = c.createInstance(Components.interfaces.INokiaPhoneInfo);
			}
		catch (e) {
			error2(strings.getString("nodll.title"),
				strings.getString("nodll.blurb1") + "\n" +
					strings.getString("nodll.blurb2") + "\n" +
					strings.getString("nodll.details"),
				e);
			close(); return;
			}

		var stat;
		try {
			nokiaPhoneInfo.Create(hResult, nmpResult);	
			stat=nokiaPhoneInfo.GetDeviceStatus(hResult, nmpResult);	
			log(1, "DEVICE STATUS " + stat);
			} 
		catch (e) {
			error2(strings.getString("noconnect.title"),
				strings.getString("noconnect.blurb1") + "\n" +
					strings.getString("noconnect.blurb2") + "\n" +
					strings.getString("noconnect.details"),
				e + "\n" + nokiaPhoneInfo.GetLastErrorString());
			close(); return;
			}
		if (stat==0)
			window.setTimeout(stepNokiaConnectDone, 0);
		else
			window.setTimeout(stepNokiaConnectRetry, 0);

	} catch (e) { errorClose2("Error in connecting to phone (connect)", e); }
	}
	

// ======================================================
/** 
	Step 3xxx: Connect to phone - RETRY
	@type void
**/
function stepNokiaConnectRetry() {
	try {
		var p=document.getElementById('progressNokiaConnect');
		p.setAttribute("hidden", "false");
		iConnect=1;
		window.setTimeout(stepNokiaConnectNext, msWait);
	} catch (e) {
		error("Error in connecting to phone (retry)", e);
		close(); return;
		}                    
	}

// ======================================================
/** 
	Try to launch the NOKIA PC Suite SMS editor
	@type void
**/
function trySMSeditor() {

	var flags=promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_0 +
		promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;
	var reply = promptService.confirmEx(window,
				strings.getString("trySMSeditor.title"),
				strings.getString("trySMSeditor.blurb1") + "\n" +
				strings.getString("trySMSeditor.blurb2") + "\n" +
				strings.getString("trySMSeditor.blurb3") + "\n" +
				strings.getString("trySMSeditor.blurb4"),
				flags,
				null, strings.getString("trySMSeditor.try"), null,
				null, {}); 
	if (reply != 1) return;
	
	try {
	  	log(1, "Trying to open SMS editor");
		
		// Get editor from registry
		var wrk = Components.classes["@mozilla.org/windows-registry-key;1"]
                .createInstance(Components.interfaces.nsIWindowsRegKey);
		// "Applications\\TextMessageEditor.exe\\shell\\Open\\command",
		wrk.open(wrk.ROOT_KEY_CLASSES_ROOT,
				"Nokia.vmgEditor.Message\\shell\\Open\\command",
				wrk.ACCESS_READ);
		var editorPath=wrk.readStringValue("");
		wrk.close();
		wrk=null;
		
		// Now a lot of work to get the filenames...
		editorPath=editorPath.replace( /\"([^\"]*)\".*/, "$1");
		var editorExe = Components.classes["@mozilla.org/filespec;1"]
                .createInstance(Components.interfaces.nsIFileSpec);
        editorExe.nativePath=editorPath;
        var ios = Components.classes["@mozilla.org/network/io-service;1"].
            getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("chrome://noksync/content/sms.vmg", null, null);
		var chrome = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
                .getService(Components.interfaces.nsIChromeRegistry);
        var smsUri=chrome.convertChromeURL(uri);
		var smsFile = Components.classes["@mozilla.org/filespec;1"]
                .createInstance(Components.interfaces.nsIFileSpec);
        smsFile.URLString=smsUri.spec;
		
		// And finally, launch the editor
        editorExe.execute(' /e "' + smsFile.nativePath + '"');

		}
	catch (e) {
		Components.utils.reportError(where + ": Unable to open SMS editor: " + e);
		var flags=promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_0;
		var reply = promptService.confirmEx(window,
					strings.getString("noSMSeditor.title"),
					strings.getString("noSMSeditor.blurb1") + "\n" +
					strings.getString("noSMSeditor.blurb2") + "\n" +
					strings.getString("noSMSeditor.blurb3") + "\n" + e,
					flags,
					null, null, null,
					null, {});
		//if (reply != 1) window.close();
		}
}


// ======================================================
/** 
	Step 3xxx: Connect to phone - NEXT
	@type void
**/
function stepNokiaConnectNext() {
	try {
		if (shouldCancel) {
			handleCancelled("NokiaConnect");
			return; }
		document.getElementById('progressNokiaConnect').setAttribute("value", 100*iConnect/nConnect);
		var stat;
		try {
			stat=nokiaPhoneInfo.GetDeviceStatus(hResult, nmpResult);	
			// log(3, "DEVICE STATUS " + stat);
		} catch (e) {
			error("Error in connecting to phone (nmp/next)", e + "\n" + nokiaPhoneInfo.GetLastErrorString());
			close(); return;
			}
		if (stat==0)
			window.setTimeout(stepNokiaConnectDone, 0);
		else {
			iConnect++;
			if (iConnect>=nConnect) {
				document.getElementById('progressNokiaConnect').setAttribute("hidden", "true");
				document.getElementById('labelNokiaConnect')
						.setAttribute("value", strings.getString("stepNokiaConnect.notConnected") );
				setStepError('NokiaConnect');
				isCancelled=true;
				document.getElementById('prompt')
						.setAttribute("value", strings.getString("prompt.connecterrorLabel") );
				}
			else {
				if (iConnect==10) trySMSeditor();
				window.setTimeout(stepNokiaConnectNext, msWait);
				}
		}
	} catch (e) {
		error("Error in connecting to phone (next)", e);
		close(); return;
		}                    
	}
	
		
// ======================================================
/** 
	Step 3xxx: Connect to phone - DONE
	@type void
**/
function stepNokiaConnectDone() {
	try {


		try {
			productType=nokiaPhoneInfo.GetProductType(hResult, nmpResult);
		} catch (e) { errorCloseNmp("Error in getting product type", e, nokiaPhoneInfo); }
		
		log(1, "PRODUCT TYPE " + productType);

		unsupported=false;
		if (!checkProductType(productType)) {
		
			var val=false;			
			if (prefBranch.getPrefType(prefSupported+productType) == prefBranch.PREF_BOOL)
				val=prefBranch.getBoolPref(prefSupported+productType);
			if (!val) {
	  			log(1, "Unsupported product type");
				var flags=promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_0 +
					promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;
				var reply = promptService.confirmEx(window,
							strings.getString("notsupported.title"),
							strings.getString("notsupported.productType") + " " + productType + "\n" +
							strings.getString("notsupported.blurb1") + "\n" +
							strings.getString("notsupported.blurb2"),
							flags,
							null, strings.getString("notsupported.continue"), null,
							null, {});
				if (reply != 1) window.close();
				unsupported=true;
				log(0, "Continuing with Unsupported product type!");
				}
			}
			/*
		try {
			nokiaPhoneInfo.Destroy(hResult, nmpResult);		
			nokiaPhoneInfo=null;
		} catch (e) {
			error("Error in connecting to phone (Destroy)", e + "\n" + nokiaPhoneInfo.GetLastErrorString());
			close(); return;
			}
			*/			
		document.getElementById('progressNokiaConnect').setAttribute("hidden", "true");
		document.getElementById('labelNokiaConnect')
				.setAttribute("value", strings.getString("stepNokiaConnect.connected") );
		// document.getElementById('iconNokiaConnect').setAttribute("src", "chrome://noksync/skin/checkmark.png");

		setStepOk("NokiaConnect");
		window.setTimeout(stepNokiaReadContacts, 0);
	} catch (e) {
		error("Error in connecting to phone (done)", e);
		close(); return;
		}                    
	}
	
		
// ======================================================
/** 
	Step 4: Read contacts from phone
	@type void
**/
function stepNokiaReadContacts() {
	try {
		if (shouldCancel) {
			handleCancelled("NokiaConnect");
			return; }

		setStepWorking("NokiaReadContacts");
		setStepLabel("NokiaReadContacts", "stepNokiaReadContacts.phonebook");
		window.setTimeout(stepNokiaReadContactsPhonebook, 0);

	} catch (e) { errorClose2("Error reading contacts from phone (init)", e); }                    
	}
	

// ======================================================
/** 
	Step 4xxx: Read contacts from phone - PHONEBOOK
	@type void
**/
function stepNokiaReadContactsPhonebook() {
	try {
		if (shouldCancel) {
			handleCancelled("NokiaReadContacts");
			return; }

		nokiaPhonebook = Components.classes["@KaarPoSoft.com/NokSync/NokiaPhonebook;1"]
				.createInstance(Components.interfaces.INokiaPhonebook);
		if (!nokiaPhonebook) throw("Could not get " + "@KaarPoSoft.com/NokSync/NokiaPhonebook;1");
			nokiaPhonebook.Create(hResult, nmpResult);	
		try {
			nContact=nokiaPhonebook.GetMemoryCapsFull(hResult, nmpResult);
		} catch (e) {
			error("Error in reading contacts from (nmp/init)", e + "\n" + nokiaPhonebook.GetLastErrorString());
			close(); return;
			}
		
		window.setTimeout(stepNokiaReadContactsFirst, 0);

	} catch (e) { errorClose2("Error reading contacts from phone (phonebook)", e); }                    
	}
	
	
// ======================================================
/** 
	Step 4xxx: Read contacts from phone - FIRST
	@type void
**/
function stepNokiaReadContactsFirst() {

try {
		iContact=1;
		document.getElementById('progressNokiaReadContacts').setAttribute("hidden", "false");
		document.getElementById('progressNokiaReadContacts').setAttribute("value", 100 * iContact / nContact);
		document.getElementById('labelNokiaReadContacts').setAttribute("value", iContact + "/" + nContact);

		try {
			var c=nokiaPhonebook.GetFirstContact(hResult, nmpResult);	
		} catch (e) {
			error("Error reading contacts from phone (nmp/first)", e + "\n" + nokiaPhonebook.GetLastErrorString());
			close(); return;
			}
		if (!c) throw("GetFirstContact returned NULL");
		var nc=new NokiaContact(c);
		aNokiaContact=[];
		aNokiaContact.push(nc);
			log(2, "FROM NOKIA: " + nc.asString());

		window.setTimeout(stepNokiaReadContactsNext, 0);

	} catch (e) { errorClose2("Error reading contacts from phone (first)", e); }                    
	}
	
	
// ======================================================
/** 
	Step 4xxx: Read contacts from phone - NEXT
	@type void
**/
function stepNokiaReadContactsNext() {

try {
		if (shouldCancel) {
			handleCancelled("NokiaReadContacts");
			return; }

		iContact++;

		// if (iContact>20) {	window.setTimeout(stepNokiaReadContactsDone, 0);	return;}

		if (iContact>nContact) {
			window.setTimeout(stepNokiaReadContactsDone, 0);	
		} else {
	
			document.getElementById('progressNokiaReadContacts').setAttribute("value", 100 * iContact / nContact);
			document.getElementById('labelNokiaReadContacts').setAttribute("value", iContact + "/" + nContact);

			try {
				var c=nokiaPhonebook.GetNextContact(hResult, nmpResult);	
			} catch (e) {
				error("Error reading contacts from phone (nmp/next)", e + "\n" + nokiaPhonebook.GetLastErrorString());
				close(); return;
				}
			if (!c) throw("GetNextContact returned NULL");
			var nc=new NokiaContact(c);
			aNokiaContact.push(nc);
			log(2, "FROM NOKIA: " + nc.asString());
			window.setTimeout(stepNokiaReadContactsNext, 0);
		}
	} catch (e) { errorClose2("Error reading contacts from phone (next)", e); }                    
	}
	
	
// ======================================================
/** 
	Step 4xxx: Read contacts from phone -  DONE
	@type void
**/
function stepNokiaReadContactsDone() {
	try {
	
		/** DON't - We need it later
			@todo remember to destroy nokiaContact later **/
		/*
		try {
			nokiaContact.Destroy(hResult, nmpResult);		
			nokiaContact=null;
		} catch (e) {
			error("Error reading contacts from phone (nmp/done)", e + "\n" + nokiaContact.GetLastErrorString());
			close(); return;
			}			
		*/
			
		document.getElementById('progressNokiaReadContacts').setAttribute("hidden", "true");
		document.getElementById('labelNokiaReadContacts')
				.setAttribute("value", strings.getFormattedString("stepNokiaReadContacts.foundContacts", [String(nContact)]) );
		setStepOk("NokiaReadContacts");
		window.setTimeout(stepSelectContacts, 0);
	} catch (e) { errorClose2("Error reading contacts from phone (done)", e); }                    
	}
	
// ======================================================
/** 
	Step 5: Select contacts to synchronize
	@type void
**/
function stepSelectContacts() {

	try {
		if (shouldCancel) {
			handleCancelled("SelectContacts");
			return; }
			
		setStepWorking("SelectContacts");
		document.getElementById('labelSelectContacts')
				.setAttribute("value", strings.getString("stepSelectContacts.comparing") );

		window.setTimeout(stepSelectContactsCompare, 0);
	} catch (e) { errorClose2("Error selecting contacts to synchronize (init)", e); }                    

	}


// ======================================================
/** 
	Step 5xxx: Select contacts to synchronize
	@type void
**/
function stepSelectContactsCompare() {

	try {
		if (shouldCancel) {
			handleCancelled("SelectContacts");
			return; }
		comparison=syncCompare(aAbCard, aCardAB, aNokiaContact);
		window.setTimeout(stepSelectContactsWin, 0);
	} catch (e) { errorClose2("Error selecting contacts to synchronize (compare)", e); }                    
	}
	
	
// ======================================================
/** 
	Step 5xxx: Select contacts to synchronize
	@type void
**/
function stepSelectContactsWin() {

	try {
		if (shouldCancel) {
			handleCancelled("SelectContacts");
			return; }

		document.getElementById('labelSelectContacts')
				.setAttribute("value", strings.getString("stepSelectContacts.selectInDialog") );
		
		var errorReturn = { cancel: false, error: false, dsTree: null };
	    window.openDialog("chrome://noksync/content/SyncSelection.xul", "", "chrome,centerscreen,resizable,modal",
			comparison, errorReturn);
			
		if (errorReturn.error) { close(); return; }
		
		if (errorReturn.cancel) {
			document.getElementById('labelSelectContacts')
					.setAttribute("value", strings.getString("stepSelectContacts.selectInDialogCancelled") );
			setStepCancel("SelectContacts");
			shouldCancel=true;
			handleCancelled("SelectContacts");
			return;
			}
			
		dsTree=errorReturn.dsTree;
		
		setStepOk("SelectContacts");

		window.setTimeout(stepSynchronize, 0);
	} catch (e) { errorClose2("Error selecting contacts to synchronize (win)", e); }                    
	}
	
	
// ======================================================
/** 
	Step 6: Synchronize
	@type void
**/
function stepSynchronize() {

	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
			
		setStepWorking("Synchronize");
		document.getElementById('labelSynchronize')
				.setAttribute("value", strings.getString("stepSynchronize.collecting") );

		var filteredComparison=new Object();
		nContacts=filterComparison(dsTree, comparison, filteredComparison);
		comparison=filteredComparison;
		
		log(1, "stepSynchronize #updateTbird = " + comparison.updateTbird.length);
		log(1, "stepSynchronize #updateNokia = " + comparison.updateNokia.length);
		log(1, "stepSynchronize #newTbird = " + comparison.newTbird.length);
		log(1, "stepSynchronize #newNokia = " + comparison.newNokia.length);
		log(1, "stepSynchronize #deleteTbird = " + comparison.deleteTbird.length);
		log(1, "stepSynchronize #deleteNokia = " + comparison.deleteNokia.length);

		if (nContacts==0) {
				document.getElementById('labelSynchronize')
						.setAttribute("value", strings.getString("stepSynchronize.nothingToSync") );
				setStepOk("Synchronize");
				window.setTimeout(stepFinished, 0);
		} else {
			var errorReturn = { cancel: false, error: false };
			window.openDialog("chrome://noksync/content/syncReady.xul", "", "chrome,centerscreen,resizable,modal",
				comparison, errorReturn);
			
			if (errorReturn.error) { close(); return; }
		
			if (errorReturn.cancel) {
				document.getElementById('labelSynchronize')
						.setAttribute("value", strings.getString("stepSynchronize.readyCancelled") );
				setStepCancel("Synchronize");
				shouldCancel=true;
				handleCancelled("Synchronize");
				return;
				}
			window.setTimeout(stepSynchronizeNewTbird, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronize", e); }                    

	}


// ======================================================
/** 
	Step 6xxx: Synchronize new in Thunderbird
	@type void
**/
function stepSynchronizeNewTbird() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		var sync=comparison.newTbird;
		var n=sync.length;
		if (n>0) {
			setStepLabelF("Synchronize", "stepSynchronize.newTbird", n);
			allNewTbird(aBooks[0], sync);
			}
		window.setTimeout(stepSynchronizeUpdateTbird, 0);
	} catch (e) { errorClose2("Error in stepSynchronizeNewTbird", e); }                    
	}
	
	
// ======================================================
/** 
	Step 6xxx: Synchronize updates to in Thunderbird
	@type void
**/
function stepSynchronizeUpdateTbird() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		var sync=comparison.updateTbird;
		var n=sync.length;
		if (n>0) {
			setStepLabelF("Synchronize", "stepSynchronize.updateTbird", n);
			allUpdateTbird(sync);
			}
		window.setTimeout(stepSynchronizeDeleteTbird, 0);
	} catch (e) { errorClose2("Error in stepSynchronizeUpdateTbird", e); }                    
	}
	
	
// ======================================================
/** 
	Step 6xxx: Synchronize delete in Thunderbird
	@type void
**/
function stepSynchronizeDeleteTbird() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		var sync=comparison.deleteTbird;
		var n=sync.length;
		if (n>0) {
			setStepLabelF("Synchronize", "stepSynchronize.deleteTbird", n);
			allDeleteTbird(sync);
			}
		window.setTimeout(stepSynchronizeNewNokia, 0);
	} catch (e) { errorClose2("Error in stepSynchronizeDeleteTbird", e); }                    
	}

	
// ======================================================
/** 
	Step 6xxx: Synchronize new on NOKIA
	@type void
**/
function stepSynchronizeNewNokia() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		nContact=comparison.newNokia.length;
		if (nContact>0) {
			iContact=0;
			document.getElementById('progressSynchronize').setAttribute("hidden", "false");
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.newNokia", (iContact+1) + "/" + nContact);
			oneNewNokia(nokiaPhonebook, comparison.newNokia[iContact]);
			window.setTimeout(stepSynchronizeNewNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeUpdateNokia, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeNewNokia", e); }                    
	}
	
	
// ======================================================
/** 
	Step 6xxx: Synchronize new on NOKIA - next
	@type void
**/
function stepSynchronizeNewNokiaNext() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		iContact++;
		if (iContact<nContact) {
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.newNokia", (iContact+1) + "/" + nContact);
			oneNewNokia(nokiaPhonebook, comparison.newNokia[iContact]);
			window.setTimeout(stepSynchronizeNewNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeUpdateNokia, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeNewNokiaNext", e); }                    
	}
	
	
// ======================================================
/** 
	Step 6xxx: Synchronize updates on NOKIA
	@type void
**/
function stepSynchronizeUpdateNokia() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		nContact=comparison.updateNokia.length;
		if (nContact>0) {
			iContact=0;
			document.getElementById('progressSynchronize').setAttribute("hidden", "false");
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.updateNokia", (iContact+1) + "/" + nContact);
			log(2, "SYNC-UPDATE-NOKIA " + iContact);
			oneUpdateNokia(comparison.updateNokia[iContact]);
			window.setTimeout(stepSynchronizeUpdateNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeDeleteNokia, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeUpdateNokia", e); }                    
	}
	

// ======================================================
/** 
	Step 6xxx: Synchronize updates on NOKIA - next
	@type void
**/
function stepSynchronizeUpdateNokiaNext() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		iContact++;
		if (iContact<nContact) {
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.updateNokia", (iContact+1) + "/" + nContact);
			oneUpdateNokia(comparison.updateNokia[iContact]);
			window.setTimeout(stepSynchronizeUpdateNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeDeleteNokia, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeUpdateNokiaNext", e); }                    
	}
	

// ======================================================
/** 
	Step 6xxx: Synchronize delete on NOKIA
	@type void
**/
function stepSynchronizeDeleteNokia() {

	function compCompare(c1, c2) {
		return c2.nokiaLocation-c1.nokiaLocation;
		}
		
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		comparison.deleteNokia.sort(compCompare);
		nContact=comparison.deleteNokia.length;
		if (nContact>0) {
			iContact=0;
			document.getElementById('progressSynchronize').setAttribute("hidden", "false");
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.deleteNokia", (iContact+1) + "/" + nContact);
			var loc=comparison.deleteNokia[iContact].nokiaLocation;
			comparison.deleteNokia[iContact]=null;
			oneDeleteNokia(loc);
			window.setTimeout(stepSynchronizeDeleteNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeDone, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeDeleteNokia", e); }                    
	}

	
// ======================================================
/** 
	Step 6xxx: Synchronize delete on NOKIA - next
	@type void
**/
function stepSynchronizeDeleteNokiaNext() {
	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }
		iContact++;
		if (iContact<nContact) {
			document.getElementById('progressSynchronize').setAttribute("value", 100 * iContact / nContact);
			setStepLabelN("Synchronize", "stepSynchronize.deleteNokia", (iContact+1) + "/" + nContact);
			var loc=comparison.deleteNokia[iContact].nokiaLocation;
			comparison.deleteNokia[iContact]=null;
			oneDeleteNokia(loc);
			window.setTimeout(stepSynchronizeDeleteNokiaNext, 0);
		} else {
			window.setTimeout(stepSynchronizeDone, 0);
			}
	} catch (e) { errorClose2("Error in stepSynchronizeDeleteNokiaNext", e); }                    
	}
	

// ======================================================
/** 
	Step 6xxx: Synchronization DOne
	@type void
**/
function stepSynchronizeDone() {

	try {
		if (shouldCancel) {
			handleCancelled("Synchronize");
			return; }

		setLastSyncDate();	
				
		document.getElementById('progressSynchronize').setAttribute("hidden", "true");
		document.getElementById('labelSynchronize')
				.setAttribute("value", strings.getFormattedString("stepSynchronize.synchronized", [String(nContacts)]) );
		setStepOk("Synchronize");

		if (unsupported) {

			// remember that users phone is supported next time		
			prefBranch.setBoolPref(prefSupported+productType, true);

	  		log(1, "Unsupported product type actually worked");
			var flags=promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_0 +
				promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_1;
			var reply = promptService.confirmEx(window,
						strings.getString("didwork.title"),
						strings.getString("didwork.productType") + " " + productType + "\n" +
						strings.getString("didwork.blurb1") + "\n" +
						strings.getString("didwork.blurb2") + "\n" +
						strings.getString("didwork.blurb3") + "\n" +  
						strings.getString("didwork.blurb4") + " " + productType,
						flags,
						null, strings.getString("didwork.report"), null,
						null, {});
			if (reply == 1) reportSupport();
			}


		window.setTimeout(stepFinished, 0);
	} catch (e) { errorClose2("Error in synchronization (done)", e); }                    
	}


// ======================================================
/** 
	Final step. Synchronization finished successfully
	@type void
**/
function stepFinished() {
			isCancelled=true;
		try {
			nokiaPhonebook.Destroy(hResult, nmpResult);		
			nokiaPhonebook=null;
		} catch (e) {
			error("Error destroying nokiaPhonebook", e);
			close(); return;
			}			
		try {
			aBooks=null;
			aAbCard=null;
			aNokiaContact=null;
		} catch (e) {
			error("Error destroying nstances", e);
			close(); return;
			}			
			document.getElementById('prompt')
				.setAttribute("value", strings.getString("prompt.finishedLabel") );
}


// ======================================================
window.addEventListener("load", function(e) { initMain(); }, false); 
