/** @file 
	@brief Class @c NokiaContact representing a contact on the NOKIA phone.	
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: nokiaContact.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

 // ======================================================

/** Global Enumeration of the different ValueTypes used in the NOKIA interface.
	@type enumeration
**/	var ContactValueTypes = {
    CONTACT_UNKNOWN : 0,
    CONTACT_NAME : 1,
    CONTACT_ADDITIONAL_NAME : 2,
    CONTACT_NOTE : 3,
    CONTACT_EMAIL_ADDRESS : 4,
    CONTACT_POST_ADDRESS : 5,
    CONTACT_VM_PASSWORD : 6,
    CONTACT_STANDARD_NUMBER : 7,
    CONTACT_HOME_NUMBER : 8,
    CONTACT_MOBILE_NUMBER : 9,
    CONTACT_FAX_NUMBER : 10,
    CONTACT_CAR_NUMBER : 11,
    CONTACT_WORK_NUMBER : 12,
    CONTACT_OFFICE_NUMBER : 13,
    CONTACT_PAGER_NUMBER : 14,
    CONTACT_VOICETAG_ID : 15,
    CONTACT_GROUP_ID : 16,
    CONTACT_MODEM_NUMBER : 17,
    CONTACT_PHONE_NUMBER : 18,
    CONTACT_RING_TONE_ID : 19,
    CONTACT_CALL_COUNT : 20,
    CONTACT_CALL_COST : 21,
    CONTACT_FORWARDING_INFO : 22,
    CONTACT_VIBRA_ID : 23,
    CONTACT_WEB_ADDRESS : 24,
    CONTACT_READING : 25,
    CONTACT_IMPS_ID : 26,
    CONTACT_SIP_ADDRESS : 27
}

/** Global holding the maximal number in @c ContactValueTypes.
	This value is used for array dimensioning.
	@type int
**/	var MaxContactValueType=27;

/** Global array of strings naming each of the different ValueTypes used in the NOKIA interface.
	@type string[]
**/	var ContactValueTypeStrings = [
    "UNKNOWN",
    "NAME",
    "ADDITIONAL_NAME",
    "NOTE",
    "EMAIL_ADDRESS",
    "POST_ADDRESS",
    "VM_PASSWORD",
    "STANDARD_NUMBER",
    "HOME_NUMBER",
    "MOBILE_NUMBER",
    "FAX_NUMBER",
    "CAR_NUMBER",
    "WORK_NUMBER",
    "OFFICE_NUMBER",
    "PAGER_NUMBER",
    "VOICETAG_ID",
    "GROUP_ID",
    "MODEM_NUMBER",
    "PHONE_NUMBER",
    "RING_TONE_ID",
    "CALL_COUNT",
    "CALL_COST",
    "FORWARDING_INFO",
    "VIBRA_ID",
    "WEB_ADDRESS",
    "READING",
    "IMPS_ID",
    "SIP_ADDRESS"
];


// ===============================================================

/** Constructor **/
function NokiaContact(xpContact) {

	function getName(hash) {
		var a=hash[ContactValueTypes.CONTACT_NAME];
		if (a.length>0) return a[0].stringValue; else return "";
		}
	
	function getHomeNumber() {
		var a=this._hash[ContactValueTypes.CONTACT_HOME_NUMBER];
		if (a.length>0) return a[0].stringValue; else return null;
		}
	
	function getEmails() {
		return this._hash[ContactValueTypes.CONTACT_EMAIL_ADDRESS];
		}

	function getAmountOfFields() {
		var n=this.xpContact.AmountOfFields(hResult, nmpResult);
		return n;
		}

	function makeFields(xxpContact) {
		var fields=[];
		var n=xxpContact.AmountOfFields(hResult, nmpResult);
		var i;
		for (i=1; i<=n; i++) {
			var valueType=new Object;
			var stringValue=new Object;
			xxpContact.GetField(i, valueType, stringValue, hResult, nmpResult);
			fields.push( { valueType: valueType.value, stringValue: stringValue.value } );
			}
			
		return fields;
		}

	function makeHash(fields) {
		var hash=[];
		for (var j=0; j<MaxContactValueType; j++) { var a=new Array; hash.push(a); }
		// The variable "i" below has typeof "String". Why ???
		for (var i in fields) { hash[fields[i].valueType].push( { field: Number(i), stringValue: fields[i].stringValue } ); }
		return hash;
		}

	function getFromValueType(valueType) {
		return this._hash[valueType];
		}

	function getFieldStringValue(fieldIdx) {
		return this._fields[fieldIdx].stringValue;
		}
	
	function getFieldValueType(fieldIdx) {
		return this._fields[fieldIdx].valueType;
		}
	
	function getTimeStamp() {
		// Don't use. Always returns Zero.
		var ts=this.xpContact.TimeStamp(hResult, nmpResult);
		//log(1, "TS=" + ts + " HRESULT=" + hResult.value + " NMPRESULT=" + nmpResult.value);
		return ts;
		}
		
	function asString() {
		var s=""
		for (var j in this._hash) {
			var a=this._hash[j];
			if (a.length>0) {
				s=s+"\n\t"+ContactValueTypeStrings[j]+"("+j+"):";
				for (var i in a)
					s=s+" ["+a[i].field+"]: "+a[i].stringValue; 
				}
			}
		return s;
		}
	
	function getLocation() {
		return this.xpContact.GetLocation(hResult, nmpResult);
		}
		
	this.xpContact=xpContact;
	this.getLocation=getLocation;
	this._fields = makeFields(xpContact);
	this._hash = makeHash(this._fields);
	this.name = getName(this._hash);
	this.getHomeNumber = getHomeNumber;
	this.getEmails = getEmails;
	this.getAmountOfFields = getAmountOfFields;
	this.getFromValueType = getFromValueType;
	this.getFieldStringValue = getFieldStringValue;
	this.getFieldValueType = getFieldValueType;
	this.getTimeStamp = getTimeStamp;
	this.asString = asString;
	}