/** @file 
	@brief Functionality for the preferences window.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: preferences.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

 // ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(preferences)";

/** Preference prefix for selected address books.
	Also defined in main.js.
	@type string
**/	var prefAbook="noksync.abook.";

// ==================================================
/** Initialize the preferences window,
	i.e. loop through address books
	@type void
**/
function initPreferences() {
	init();
	log(1, "Initializing Preferences");
	try {
		var abox=document.getElementById('abooks');
		var preferences=document.getElementById('preferences');
		var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);
		var abDirectory = Components.classes["@mozilla.org/rdf/resource-factory;1?name=moz-abdirectory"]
							.getService(Components.interfaces.nsIAbDirectory);
		var cn = abDirectory.childNodes;
		
		while (cn.hasMoreElements() ) {
			var ab = cn.getNext();
			var curDir = ab.QueryInterface(Components.interfaces.nsIAbDirectory);
			var abName=curDir.dirName;
			log(2, "abDirectory: " + abName);
			var preference = document.createElement("preference");
			preference.setAttribute("id", abName);
			preference.setAttribute("type", "bool");
			preference.setAttribute("name", prefAbook+abName);
			preferences.appendChild(preference);
			var checkbox = document.createElement("checkbox");
			checkbox.setAttribute("id", "cb"+abName);
			checkbox.setAttribute("label", abName);
			checkbox.setAttribute("preference", abName);
			var val=false;			
			if (prefBranch.getPrefType(prefAbook+abName) == prefBranch.PREF_BOOL)
				val=prefBranch.getBoolPref(prefAbook+abName);
			checkbox.setAttribute("checked", val);
			abox.appendChild(checkbox);
		}
		var pane=document.getElementById('pane');
		var prefwindow=document.getElementById('prefwindow');
		prefwindow.showPane(pane);
	log(1, "Initialized Preferences");
	} catch (e) {
		error("Unable to show address-books in preferences", e);
	  }
}

window.addEventListener("load", function(e) { initPreferences(); }, false); 
