/** @file 
	@brief Functionality for showing number of contacts to synchronize.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: syncReady.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(SyncReady)";


// ======================================================

/** Global bundle of strings to be used on the GUI.
	Initialized by @c initSyncSelection();
	@type stringbundle
**/	var stringBundle;

/** Global. Result of comparing Thunderbird with NOKIA
	Passed as window.arguments[0] to @c initSyncReady();
	@type Comparison
**/ var comparison;

// ==================================================
/** Set all global variables needed.
	Called due to the addEventListener.
	@type void
**/
function initSyncReady() {
	init();
	log(1, "Initializing(SyncReady)");

	try {		

		stringBundle = document.getElementById("noksync-strings");

		comparison=window.arguments[0];
		errorReturn=window.arguments[1];
		
		var types=["deleteTbird", "deleteNokia", "updateTbird", "updateNokia", "newTbird", "newNokia"];
		var tosynchronize = document.getElementById("tosynchronize");
		for (i in types) {
			var c=eval("comparison."+types[i]);
			var n=c.length;
			if (n>0) {
				var s=stringBundle.getFormattedString("syncready." + types[i], [String(n)] )
				var label=document.createElement("label");
				label.setAttribute("value", s);
				tosynchronize.appendChild(label);
				}
			}	
		
		log(1, "Initialized(SyncReady)");
		
	} catch (e) { errorClose("Error in SyncReady", e, errorReturn); }
	}



// ======================================================
/** Event handler for "OK" / "Synchronize" button
	@type void
**/
function execute() {
	errorReturn.cancel=false;
	window.close()
	}

// ======================================================
/** Event handler for "Cancel" button
	@type void
**/
function cancel() {
	errorReturn.cancel=true;
	window.close()
	}
	

// ======================================================
window.addEventListener("load", function(e) { initSyncReady(); }, false); 
