/** @file 
	@brief Functionality for the SyncSelection window.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: syncSelection.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(SyncSelection)";


// ======================================================

/** Global bundle of strings to be used on the GUI.
	Initialized by @c initSyncSelection();
	@type stringbundle
**/	var stringBundle;

/** The branch of the preference tree, in which NokSync preferences lives.
	Initialized by @c initSyncSelection();
	@type nsIPrefBranch
**/	var prefBranch;

/** Global. Result of comparing Thunderbird with NOKIA
	Passed as window.arguments[0] to @c initSyncSelection();
	@type Comparison
**/ var comparison;

/** Global. RDF datasource for synctree.
	Initialized by @c initSyncSelection();
	@type nsIRDFDataSource
**/ var dsTree;

/** Global. RDF datasource for synclistbox.
	Initialized by @c initSyncSelection();
	@type nsIRDFDataSource
**/ var dsListbox;


// ======================================================

/** Global. The tree on the left in the window
	Initialized by @c initSyncSelection();
	@type XUL::tree
**/	var synctree;

/** Global. The listbox on the right in the window
	Initialized by @c synctreeSelect();
	@type XUL::listbox
**/	var synclistbox;

/** Global. The label at the top of the right of the window
	Initialized by @c synctreeSelect();
	@type XUL::label
**/	var itemtoplabel;

/** Global. The checkbox for selecting "dont synchronize..."
	Initialized by @c synctreeSelect();
	@type XUL::checkbox
**/	var checkboxdont;

/** Global. The checkbox for selecting "never synchronize..."
	Initialized by @c synctreeSelect();
	@type XUL::checkbox
**/	var checkboxnever;

/** Global. The button for selecting "update ... instead"
	Initialized by @c synctreeSelect();
	@type XUL::button
**/	var buttonopposite;

/** Global. The label for selecting "update ... instead"
	Initialized by @c synctreeSelect();
	@type XUL::label
**/	var labelopposite;

/** Global. Array indicating the opposite of some action.
	Used for labelling "update ... instead"
	@type string[]
**/	var oppositeType = [];
oppositeType["updateTbird"]="updateNokia";
oppositeType["updateNokia"]="updateTbird";
oppositeType["newTbird"]="deleteNokia";
oppositeType["newNokia"]="deleteTbird";
oppositeType["deleteTbird"]="newNokia";
oppositeType["deleteNokia"]="newTbird";


// ======================================================

/** Global. Type of the item selected in synctree
	@type string
**/	var selectedType;

/** Global. Index (given the type) of the item selected in synctree
	@type int
**/	var selectedIdx;


// ======================================================

/** Global RDF service
	Initialized by @c initSyncSelection();
	@type nsIRDFService
**/	var rdfService;

/** Global. Predicate "#Properties".
	Initialized by @c initSyncSelection();
	@type nsIRDFResource
**/	var predicateProperties;

/** Global. Predicate "#What".
	Initialized by @c initSyncSelection();
	@type nsIRDFResource
**/	var predicateWhat;

/** Global. Predicate "#li".
	Initialized by @c initSyncSelection();
	@type nsIRDFResource
**/	var predicateListItem;

/** Global. Literal "dontsync".
	Initialized by @c initSyncSelection();
	@type nsIRDFLiteral
**/	var literalPropertiesDontSync;

/** Global. Object with two properties: error and cancel.
	Those are set to true before returning, if appropriate
	Initialized by @c initSyncSelection();
	Passed as window.arguments[1] to @c initSyncSelection();
	@type Object
**/	var errorReturn;


// ======================================================
//		RDF functions - only for debug
// ======================================================

/** for debug **/
var outputStream = {
  data: "",
  close : function(){},
  flush : function(){},
  write : function (buffer,count){
    this.data += buffer;
    return count;
  },
  writeFrom : function (stream,count){},
  isNonBlocking: false
}

/** for debug **/
function serializeForDebug(ds) {
	var serializer=Components.classes["@mozilla.org/rdf/xml-serializer;1"]
                        .createInstance(Components.interfaces.nsIRDFXMLSerializer);
	serializer.init(ds);
	serializer.QueryInterface(Components.interfaces.nsIRDFXMLSource);
	serializer.Serialize(outputStream);
	log(1, outputStream.data);
}


// ==================================================
//		INITIALIZATION
// ==================================================

/** @todo: The dialog window stays open, even if application is closed **/
/*
var quitObserver = {
  observe: function(subject,topic,data){
	//log(1, "WWW " + window + "  "  + window.close);
    window.close();
	//return true;
	return false;
  }
};
*/


// ==================================================
/** Set all global variables needed.
	Called due to the addEventListener.
	@type void
**/
function initSyncSelection() {
	init();
	log(1, "Initializing(SyncSelection)");

	try {		

		stringBundle = document.getElementById("noksync-strings");

  		prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

		rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
				getService(Components.interfaces.nsIRDFService);

		predicateProperties = rdfService.GetResource(NS+"#Properties");
		predicateWhat = rdfService.GetResource(NS+"#What");
		predicateListItem = rdfService.GetResource(NSRDF+"#li");
		literalPropertiesDontSync=rdfService.GetLiteral("dontsync");

		comparison=window.arguments[0];
		errorReturn=window.arguments[1];
		initComparator();
		
		dsTree=getRDFAllSyncs(comparison);
		recount(dsTree);
		errorReturn.dsTree=dsTree;
		
		// serializeForDebug(dsTree);
		
		
        synctree = document.getElementById('synctree');
		synctree.database.AddDataSource(dsTree);
		synctree.builder.rebuild();
		
        synclistbox = document.getElementById('synclistbox');

	    checkboxdont = document.getElementById('dont');
	    checkboxnever = document.getElementById('never');
	    buttonopposite = document.getElementById('opposite');
	    labelopposite = document.getElementById('oppositeLabel');

	    itemtoplabel = document.getElementById('itemtoplabel');
		itemviewNone();


		/** @todo: The dialog window stays open, even if application is closed **/
		/*
		var observerService =
			Components.classes["@mozilla.org/observer-service;1"]
				.getService(Components.interfaces.nsIObserverService);
		observerService.addObserver(quitObserver,"quit-application-requested",false);
		*/
		
		log(1, "Initialized(SyncSelection)");
		
	} catch (e) { errorClose("Error in initSyncSelection", e, errorReturn); }
	}


// ==================================================
//		HELPERS FOR EVENT HANDLERS
// ==================================================


/** Find indices into @c comparison for the given RDF resource 
	@tparam nsIRDFResource rdfRes
	@type Selection
**/
function getSelected(rdfRes) {
	var rdfResVal=rdfRes.Value;
	var i=rdfResVal.lastIndexOf("/");
	var s=rdfResVal.substring(i+1);
	var j=s.indexOf(".");
	
	selected= { type: s.substring(0, j),
				idx: s.substring(j+1) 
				}
	return selected;
	}
	

// ==================================================
/** Rename all elements with source @c rdfOld to @c rdfNew
	@tparam nsIRDFDataSource ds
	@tparam nsIRDFResource rdfOld
	@tparam nsIRDFResource rdfNew
	@type void
**/
function renameRdfResource(ds, rdfOld, rdfNew) {
	var predicates = ds.ArcLabelsOut(rdfOld);
	while (predicates.hasMoreElements()){
		var predicate = predicates.getNext();
		var targets = ds.GetTargets(rdfOld, predicate, true);
		while (targets.hasMoreElements()){
			var target = targets.getNext();
			ds.Move(rdfOld, rdfNew, predicate, target);
			}
		}
	}

// ==================================================
/** Count number of items selected in the left treeview,
	and update parent rows accordingly.
	@tparam nsIRDFDataSource ds
	@type void
**/
function recount(ds) {
	var types=["updateTbird", "updateNokia", "newTbird", "newNokia", "deleteTbird", "deleteNokia"];
	for (i in types) {
		var rdfRes = rdfService.GetResource(NS+types[i]);
		var container = Components.classes["@mozilla.org/rdf/container;1"].
				createInstance(Components.interfaces.nsIRDFContainer);
		container.Init(ds, rdfRes);
		var n=0;
		var m=0;
		var children=container.GetElements()
		while (children.hasMoreElements()) {
			var child=children.getNext();
			n=n+1;
			var dont=dsTree.hasArcOut(child, predicateProperties);
			if (!dont) m=m+1;
			}
		var oldLiteral = ds.GetTarget(rdfRes, predicateWhat, true);
		var newLiteral = rdfService.GetLiteral(stringBundle.getFormattedString("comparator." + types[i], [String(m), String(n)] ));
		ds.Change(rdfRes, predicateWhat, oldLiteral, newLiteral);
		}
	}


// ==================================================
/** Show no selected item on the right
	@type void
**/
function itemviewNone() {
	log(1, "itemviewNone");
	if (dsListbox) synclistbox.database.RemoveDataSource(dsListbox);
	synclistbox.builder.rebuild();
	itemtoplabel.value=stringBundle.getString("item.toplabel.default");
	checkboxdont.label=stringBundle.getString("item.dont.default");
	checkboxnever.label=stringBundle.getString("item.never.default");
	labelopposite.value=stringBundle.getString("item.opposite.default");
	checkboxdont.checked=false;
	checkboxnever.checked=false;
	checkboxdont.disabled=true;
	checkboxnever.disabled=true;
	buttonopposite.disabled=true;
	labelopposite.disabled=true;
	}


// ======================================================
/** Show the selected item on the right
	@tparam string selectedType
	@tparam int selectedIdx
	@tparam Comparison comparison
	@tparam nsIRDFResource rdfRes The RDF node at the item selected in tree
	@type void
**/
function itemview(selectedType, selectedIdx, comparison, rdfRes) {
		var abCard=eval("comparison."+selectedType+"[selectedIdx].tbird");
		var nokiaContact=eval("comparison."+selectedType+"[selectedIdx].nokia");
		var aFieldIdx=eval("comparison."+selectedType+"[selectedIdx].aFieldIdx");
		log(1, "itemview abCard=" + abCard + " nokiaContact=" + nokiaContact);
		if (dsListbox) synclistbox.database.RemoveDataSource(dsListbox);
		dsListbox=getRDFitem(abCard, nokiaContact, aFieldIdx);
		synclistbox.database.AddDataSource(dsListbox);
		synclistbox.builder.rebuild();
		itemtoplabel.value=stringBundle.getString("item.toplabel."+selectedType);
		checkboxdont.label=stringBundle.getString("item.dont."+selectedType);
		checkboxnever.label=stringBundle.getString("item.never."+selectedType);
		labelopposite.value=stringBundle.getString("item.opposite."+selectedType);
		var dont=dsTree.hasArcOut(rdfRes, predicateProperties);
		checkboxdont.checked=dont;
		var never=false;
		if (dont) {
			
			buttonopposite.disabled=true;
			labelopposite.disabled=true;

			var name=dsTree.GetTarget(rdfRes, predicateWhat, true).Value;
			var pref="extensions.noksync.dont."+name;
			if (prefBranch.prefHasUserValue(pref))
				if (prefBranch.getPrefType(pref) == prefBranch.PREF_BOOL)
					never=prefBranch.getBoolPref(pref);
			}
		checkboxnever.checked=never;
		var dis=false;
		if ( (selectedType=="ignored") || (selectedType=="matching") ) dis=true;
		checkboxdont.disabled=dis;
		checkboxnever.disabled=dis;
		if (dont) dis=true; 
		buttonopposite.disabled=dis;
		labelopposite.disabled=dis;
}


// ==================================================
//		EVENT HANDLERS
// ==================================================

/** Event handler for checkbox "Never synchronize..."
	@tparam nsIDOMEvent event
	@type void
**/
function neverSelected(event) {
	try {
		var target=event.currentTarget;
		log ("neverSelected: " + target.checked);
		var idx=synctree.currentIndex;
		var rdfRes=synctree.view.getResourceAtIndex(idx);
		var name=dsTree.GetTarget(rdfRes, predicateWhat, true).Value;
		var pref="extensions.noksync.dont."+name
		if (target.checked) {
			prefBranch.setBoolPref(pref, true);
			// if never, also dont
			checkboxdont.checked=true;
			dsTree.Assert(rdfRes, predicateProperties, literalPropertiesDontSync, true);
			// and also do not do the opposite
			buttonopposite.disabled=true;
			labelopposite.disabled=true;
		} else {
			prefBranch.clearUserPref(pref);
			}
		recount(dsTree);
	} catch (e) { errorClose("Error in neverSelected", e, errorReturn); }
	}
		

// ==================================================
/** Event handler for checkbox "Dont synchronize..."
	@tparam nsIDOMEvent event
	@type void
**/
function dontSelected(event) {
	try {
		var target=event.currentTarget;
		log ("dontSelected: " + target.checked);
		
		// update the RDF datasource to show gray or not, according to checkbox
		var idx = synctree.currentIndex;
		var rdfRes=synctree.view.getResourceAtIndex(idx);
		if (target.checked) {
			// checkbox checked, i.e. item not selected
			// so add properties
			dsTree.Assert(rdfRes, predicateProperties, literalPropertiesDontSync, true);
			// so, we can not do the opposite
			buttonopposite.disabled=true;
			labelopposite.disabled=true;
		} else {
			// checkbox not checked, i.e. item selected
			// so remove the properties
			var literalTarget=dsTree.GetTarget(rdfRes, predicateProperties, true);
			if  (literalTarget) dsTree.Unassert(rdfRes, predicateProperties, literalTarget);
			// and remove "never"
			checkboxnever.checked=false;
			var name=dsTree.GetTarget(rdfRes, predicateWhat, true).Value;
			var pref="extensions.noksync.dont."+name;
			if (prefBranch.prefHasUserValue(pref))
				prefBranch.clearUserPref(pref);
			// and now we can do the opposite
			buttonopposite.disabled=false;
			labelopposite.disabled=false;
			}
		recount(dsTree);
			
		// was playing around with getting the property directly into the tree (without RDF)
		// but never got it to work
		/*
		var prop=new Object;
		var col=synctree.columns.getColumnAt(0);
		synctree.view.getCellProperties ( ci , col , prop);		
		// http://www.xulplanet.com/tutorials/xultu/treestyle.html
		// PROBABLY NEED A QUERYINTERFACE HERE....
		log(1, "PROP ", prop);
	    prop.value.AppendElement("dont");
		*/

	} catch (e) { errorClose("Error in dontSelected", e, errorReturn); }
	}


// ==================================================
/** Event handler for button "Update on ... instead"
	@tparam nsIDOMEvent event
	@type void
**/
function oppositeSelected(event) {
	try {
		var target=event.currentTarget;
		log ("oppositeSelected");
		
		// find current RDF resource
		var idx = synctree.currentIndex;
		var rdfRes=synctree.view.getResourceAtIndex(idx);
		
		// find paretnt RDF resource
		var parentIdx=synctree.view.getParentIndex(idx);
		var rdfParent=synctree.view.getResourceAtIndex(parentIdx);
		var container = Components.classes["@mozilla.org/rdf/container;1"].
				createInstance(Components.interfaces.nsIRDFContainer);
		container.Init(dsTree, rdfParent);

		// set properties, so it will not be synchronized		
 		//dsTree.Assert(rdfRes, predicateProperties, literalPropertiesDontSync, true);
 		
		// remove current element, so it will not show up in synctree on the left
		container.RemoveElement(rdfRes, true);

		// find the opposite selection type
		var selected=getSelected(rdfRes);
		opposite=oppositeType[selected.type];

		// add compared element to opposite comparison
		var elem=eval("comparison."+selected.type+"["+selected.idx+"]");
		eval("comparison."+opposite).push(elem);
		
		// null out the compared element
		eval("comparison."+selected.type+"["+selected.idx+"]=null");

		// create opposite element
		var n=eval("comparison."+opposite).length-1;
		//rdfRes.Value=NS+opposite+"." + n;
		//var resOpposite=rdfRes;
		var resOpposite = rdfService.GetResource(NS+opposite+"." + n);
		renameRdfResource(dsTree, rdfRes, resOpposite);

		// find container for opposite selection type
		rdfOpposite = rdfService.GetResource(NS+opposite);
		var oppositeContainer = Components.classes["@mozilla.org/rdf/container;1"].
				createInstance(Components.interfaces.nsIRDFContainer);
		oppositeContainer.Init(dsTree, rdfOpposite);
			
		// add opposite element to opposite container
		oppositeContainer.AppendElement(resOpposite);

		recount(dsTree);
		
	} catch (e) { errorClose("Error in oppositeSelected", e, errorReturn); }
	}


// ======================================================
/** Event handler for selection of event on the left
	@tparam nsIDOMEvent event
	@type void
**/
function synctreeSelect(event) {
	try {		
		var v=synctree.view;
		var idx=synctree.currentIndex;
		log(1, "synctreeSelect idx=" + idx);
		if (idx<0) { itemviewNone(); return; }
		var rdfRes=v.getResourceAtIndex(idx);
		var rdfResVal=rdfRes.Value;
		var i=rdfResVal.lastIndexOf("/");
		var s=rdfResVal.substring(i+1);
		var j=s.indexOf(".");
		
		selectedType=s.substring(0, j);
		selectedIdx=s.substring(j+1);
		
		// promptService.alert(window, "Select", "select " + event.currentTarget.id + " " + idx + " " +  rdfResVal + " XXX " + s + " WHAT " + selectedType + " WHERE " + selectedIdx);
		log(1, "synctreeSelect: selectedType=" + selectedType + " selectedIdx=" + selectedIdx);
		if (selectedType)
			// a contact
			itemview(selectedType, selectedIdx, comparison, rdfRes);
		else
			// an outer row, not a contact
			itemviewNone();
	} catch (e) { errorClose("Error in synctreeSelect", e, errorReturn); }
	}


// ==================================================
/** Event handler for right click on the left treeview
	@tparam nsIDOMEvent event
	@type void
**/
function notThisPopup(event) {
	try {		
		// consider putting this into a function, it is copy from synctreeSelect
		log(1, "notThisPopup");
		var v=synctree.view;
		var idx=synctree.currentIndex;
		log(1, "notThisPopup idx="+idx);
		var rdfRes=v.getResourceAtIndex(idx);
		var level=v.getLevel(idx)
		log(1, "notThisPopup level="+level);
		if (level>0) {
			// level>0 so this is a contact item
			checkboxdont.checked=true;
			// so add properties
			dsTree.Assert(rdfRes, predicateProperties, literalPropertiesDontSync, true);
			// so, we can not do the opposite
			buttonopposite.disabled=true;
			labelopposite.disabled=true;
			}
		else {
			// level=0 so this is a container
			var container = Components.classes["@mozilla.org/rdf/container;1"].
                  createInstance(Components.interfaces.nsIRDFContainer);
			container.Init(dsTree, rdfRes);
			var targets=container.GetElements();
			while (targets.hasMoreElements()) {
				var child=targets.getNext();
				log(3, "child: " + child);
				dsTree.Assert(child, predicateProperties, literalPropertiesDontSync, true);				
				}
			}
		recount(dsTree);
	} catch (e) { errorClose("Error in notThisPopup", e, errorReturn); }
	}


// ======================================================
/** Event handler for "OK" / "Synchronize" button
	@type void
**/
function execute() {
	tbirdAb=null;
	nokiaPb=null;
	window.close()
	//return true;
	}

// ======================================================
/** Event handler for "Cancel" button
	@type void
**/
function cancel() {
	errorReturn.cancel=true;
	window.close()
	//return true;
	}
	

// ======================================================
/** Event handler for "Help" button
	@type void
**/
function help() {
	openURL("http://www.KaarPoSoft.dk/noksync/manual.html");
	}


// ======================================================
window.addEventListener("load", function(e) { initSyncSelection(); }, false); 
