/** @file 
	@brief Functionality for synchronizing Thunderbird with NOKIA.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: synchronizer.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(Synchronizer)";


// ======================================================
/** Filter comparison according to "dont" in datasource.
	@tparam nsIRDFDataSource dsTree
	@tparam Comparison oldComparison
	@tparam Comparison filteredComparison Output. Must be initialized to <code>new Object();</code> before call.
	@treturn int Number of contacts to synchronize.
**/
function filterComparison(dsTree, oldComparison, filteredComparison) {

	function filterOneType(type) {
		var old=eval("oldComparison."+type);
		var filtered=eval("filteredComparison."+type);
		for (var i in old) {
			// if an item has been changed to opposite direction, don't synchronize "old" direction
			if (!old[i]) continue;
			// if an item has properties (i.e. grayed out) this means the item should not be synchronized
			var rdfRes = rdfService.GetResource(NS+type+"." + i);
			var dont=dsTree.hasArcOut(rdfRes, predicateProperties);
			if (!dont) filtered.push(old[i]);
			}
		}
	
	var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
				getService(Components.interfaces.nsIRDFService);
	var predicateProperties = rdfService.GetResource(NS+"#Properties");

	filteredComparison.updateTbird=[];
	filteredComparison.updateNokia=[];
	filteredComparison.newTbird=[];
	filteredComparison.newNokia=[];
	filteredComparison.deleteTbird=[];
	filteredComparison.deleteNokia=[];
	
	filterOneType("updateTbird");
	filterOneType("updateNokia");
	filterOneType("newTbird");
	filterOneType("newNokia");
	filterOneType("deleteTbird");
	filterOneType("deleteNokia");
	
	var n=0;
	n=n+filteredComparison.updateTbird.length;
	n=n+filteredComparison.updateNokia.length;
	n=n+filteredComparison.newTbird.length;
	n=n+filteredComparison.newNokia.length;
	n=n+filteredComparison.deleteTbird.length;
	n=n+filteredComparison.deleteNokia.length;
	return n;	
}


// ======================================================
/** Create all new Thunderbird contacts
	@tparam nsIAbDirectory abDirectory
	@tparam Comparison aNew
	@type void
**/
function allNewTbird(abDirectory, aNew) {
	for (i in aNew) {
		var nokiaContact=aNew[i].nokia;
		log(2, "allNewTbird: " + nokiaContact.name);
		log(3, nokiaContact.asString);
		var abCard = Components.classes["@mozilla.org/addressbook/cardproperty;1"].
				createInstance(Components.interfaces.nsIAbCard);

		var nokiaUsed=new Array(nokiaContact.getAmountOfFields());
		item_loop:
		for (var i=0; i<nItems; i++) {
			var aCVT=aNokiaCVT[i];
			for each (var cvt in aCVT) {
				var aField=nokiaContact.getFromValueType(cvt);
				for each (var f in aField) {
					if (!nokiaUsed[f.field]) {
						nokiaUsed[f.field]=true;
						abCard[aTbirdField[i]]=f.stringValue
						continue item_loop;
			}}}}

		// unmatched phone numbers
		for each (var cvt in aNokiaMainPhoneCVT.concat(aNokiaOtherPhoneCVT)) {
			var aField=nokiaContact.getFromValueType(cvt);
			field_loop:
			for each (var f in aField) {
				if (nokiaUsed[f.field]) continue;
				for (var i=0; i<nItems; i++) {
					if (aTbirdIsPhone[i] && (!abCard[aTbirdField[i]]) ) {
						nokiaUsed[f.field]=true;
						abCard[aTbirdField[i]]=f.stringValue
						continue field_loop;
						}
		}}}
		abDirectory.addCard(abCard);
		}
	}
	
// ======================================================
/** Update all selected Thunderbird contacts
	@tparam Comparison aUpdate
	@type void
**/
function allUpdateTbird(aUpdate) {
	for (i in aUpdate) {
		var abCard=aUpdate[i].tbird;
		log(2, "allUpdateTbird: " + abCard.displayName);
		var nokiaContact=aUpdate[i].nokia;
		var aFieldIdx=aUpdate[i].aFieldIdx;
		var abName=aUpdate[i].tbirdABname;
		for (var i in aTbirdField) {
			j=aFieldIdx[i];
			if (j>-1)
				abCard[aTbirdField[i]]=nokiaContact.getFieldStringValue(j);
			else if (abCard[aTbirdField[i]]) abCard[aTbirdField[i]]="";
			}
		abCard.editCardToDatabase("moz-abmdbdirectory://"+abName);		
		}
	}
	
	
// ======================================================
/** Delete all selected Thunderbird contacts
	@tparam Comparison aDelete
	@type void
**/
function allDeleteTbird(aDelete) {
	for each (var del in aDelete) {
		log(2, "allDeleteTbird " + del.tbirdAB.dirName + " " + del.tbird.displayName);
		// nsiAbDirectory.dropCard does not seem to work.
		// in fact, it seems to DUPLICATE the card.
		// So, instead, emulate an nsISupportsArray object
		var a=new Object();
		a.Count=function(){return 1};
		a.QueryElementAt=function(x){return del.tbird};
		del.tbirdAB.deleteCards(a);
		}
	}
	
// ======================================================
/** Strip unwanted characters off a phone number
	@tparam string number 
	@type void
**/
function stripNumber(number) {
	var stripped=number.replace(/[^0-9\*\+\pw]/g, "");
	return stripped;
	}
	
// ======================================================
/** Create one new NOKIA contact
	@tparam INokiaPhonebook nokiaPhonebook
	@tparam Comparison theNew
	@type void
**/
function oneNewNokia(nokiaPhonebook, theNew) {
	var abCard=theNew.tbird;
	var rawContact;
	try { rawContact=nokiaPhonebook.CreateContact(hResult, nmpResult); }
	catch (e) { errorDisplayThrowNmp("oneNewNokia:CreateContact", e, nokiaPhonebook); }
//	rawContact.AddField(ContactValueTypes.CONTACT_NAME, "Abc test", hResult, nmpResult);

	for (var i in aTbirdField) {
		s=abCard[aTbirdField[i]];
		if (s) {
			if (aTbirdIsPhone[i]) s=stripNumber(s);
			log(3, "ADDFIELD " + aTbirdField[i] + " " + aNokiaCVT[i][0] + " " + s);
			try { rawContact.AddField(aNokiaCVT[i][0], s, hResult, nmpResult); }
			catch (e) { errorDisplayThrowNmp("oneNewNokia:AddField", e, rawContact); }
			}
		}
	try { nokiaPhonebook.SetContact(0, rawContact, hResult, nmpResult); }
	catch (e) { errorDisplayThrowNmp("oneNewNokia:SetContact", e, nokiaPhonebook); }
}

	
// ======================================================
/** Update one NOKIA contact
	@tparam Comparison toUpdate
	@type void
**/
function oneUpdateNokia(toUpdate) {

function reverseCompareNumbers(a, b) {
   return b-a
	}
	var abCard=toUpdate.tbird;
	var nokiaContact=toUpdate.nokia;
	var aFieldIdx=toUpdate.aFieldIdx;

	log(2, "UPDATENOKIA " + abCard.displayName);
	
	// Remember which fields to delete
	var toDelete=[];
	
	for (var i in aTbirdField) {
		var tb=abCard[aTbirdField[i]];
		if (aTbirdIsPhone[i]) tb=stripNumber(tb);
		var idx=aFieldIdx[i];
		log(3, "UPDATENOKIA I IDX " + i + " " + idx + " " + typeof(idx));
		if (tb) {
			// There is a value in Thunderbird
			if (idx>=0) {
				// There is also a value in NOKIA
				var nok=nokiaContact.getFieldStringValue(idx);
				if (nok != tb) {
					// The value in Thunderbird and NOKIA are different, so update
					log(3, "UPDATENOKIA update " + i + " " + idx + " " + nok + " -> " + tb);
					try { nokiaContact.xpContact.SetField(idx+1, String(tb), hResult, nmpResult); }
					catch (e) { errorDisplayThrowNmp("oneUpdateNokia:SetField", e, nokiaContact.xpContact); }

					}
				}
			else {
				// There is NOT also a value in NOKIA, so add
				log(3, "UPDATENOKIA add " + i + " " + idx + " " + tb);
				try { nokiaContact.xpContact.AddField(aNokiaCVT[i][0], String(tb), hResult, nmpResult); }
				catch (e) { errorDisplayThrowNmp("oneUpdateNokia:AddField", e, nokiaContact.xpContact); }
				}
			}
		else {
			// There is NO value in Thunderbird
			if (idx>=0) {
				var nok=nokiaContact.getFieldStringValue(idx);
				// But there IS a value in NOKIA, so delete
				log(3, "UPDATENOKIA mark delete " + i + " " + idx + " " + nok);
				toDelete.push(idx);
				}
			}
		}

	// Must delete fields last, in reverse order,
	// otherwise renumbering of fields would cause trouble
	toDelete.sort(reverseCompareNumbers);
	for each (var idx in toDelete) {
		log(3, "UPDATENOKIA really delete " + idx);
		var ii=idx+1;
		log(3, "UPDATENOKIA ii=" + ii);
		try { nokiaContact.xpContact.DeleteField(ii, hResult, nmpResult); }
		catch (e) { errorDisplayThrowNmp("oneUpdateNokia:DeleteField", e, nokiaContact.xpContact); }
		}
	
	try { nokiaPhonebook.UpdateContact(nokiaContact.xpContact, hResult, nmpResult); }
	catch (e) { errorDisplayThrowNmp("oneUpdateNokia:UpdateContact", e, nokiaPhonebook); }

}

// ======================================================
/** Delete one NOKIA contact
	@tparam long locationToDelete Location in the NOKIA phone to delete
	@type void
**/
function oneDeleteNokia(locationToDelete) {
	log(0, "DELETENOKIA " + locationToDelete);
	nokiaPhonebook.DeleteContact(Number(locationToDelete), hResult, nmpResult);			
}
